# frozen_string_literal: true

#
module LostInTranslation
  def self.prepare_for_edit(options = {})
    paths = tmp_paths
    FileUtils.cp(options[:master], paths[:master]) if options[:master]
    FileUtils.cp(options[:slave], paths[:slave]) if options[:slave]
  end

  def self.prepare_paths(paths = [])
    paths.each do |p|
      prepare_yaml(p, false)
    end
  end

  def self.postpare_paths(paths = [])
    paths.each do |p|
      prepare_yaml(p, true)
    end
  end

  def self.define_snippets
    snippets = []
    snippets << ['<<', 'a_greater_than_sign']
    snippets << ['*', 'an_asterik_sign']
    snippets << ['!', 'a_bang_sign']
    snippets << ['%', 'a_percentage_sign']
    snippets << ['a_bang_sign \'', '\'a_bang_sign']
    snippets
  end

  def self.reset_tmp_files
    paths = tmp_paths
    paths.each_value do |path|
      File.open(path, 'w') { |file| file.write('') }
    end
  end

  private

  def self.prepare_yaml(path, post)
    text = File.read(path)
    snippets = define_snippets

    snippets = snippets.reverse if post
    snippets.each do |snippet|
      snippet = snippet.reverse if post
      text = text.gsub(snippet.first, snippet.second)
    end
    text = post ? postpare(snippets, text) : prepare(snippets, text)

    File.open(path, 'w') { |file| file.puts text }
  end

  def self.postpare(variables, text)
    variables = text.scan(/varyberryterry.*/)
    variables.each do |var|
      new_var = var.delete(':')
      new_var = new_var.gsub('varyberryterry', ': &')
      text = text.gsub(var, new_var)
    end
    text = add_single_quotes(text)
    text = text.gsub(': !,', ": ! ','")
    text = text.gsub('\'!', '! \'')
    text = text.gsub(/---\n/, '')
    text = text.gsub('!&hellip;', "! '&hellip;'")
    text = text.gsub('!:distance_in_words', "! ':distance_in_words'")
    text
  end

  def self.prepare(variables, text)
    variables = text.scan(/: &.*/)
    variables.each do |var|
      new_var = var.gsub(': &', 'varyberryterry')
      new_var += ':'
      text = text.gsub(var, new_var)
    end
    text
  end

  def self.add_single_quotes(text)
    tmp = ''
    text.each_line do |line|
      tmp += edit_line(line)
    end
    tmp
  end

  def self.edit_line(line)
    return line if line.match(/: .*%.*$/).blank?
    arr = line.split(': ', 2)
    return line if arr[1].blank?
    arr[1] = arr[1].gsub("\'", '')
    arr[1] = arr[1].gsub("\"", '')
    arr[1] = "'" + arr[1] unless arr[1].start_with?("\'") || arr[1].start_with?("\"")
    arr[1] = arr[1].squish + "'\n" unless arr[1].squish.end_with?("\'") || arr[1].squish.end_with?("\"")
    arr[1] = arr[1].gsub("\'! ", "! \'")
    return arr.join(': ')
  end
end
