# frozen_string_literal: true

#
module LostInTranslation
  require 'readline'

  def self.ask_for_file(master, namespace = '')
    text = master ? "# Master-Locale (e.g. 'en'): " : "# Slave-Locale (e.g. 'de'): "
    file = {}
    file[:lang] = [(print text), Readline.readline()][1]
    file[:filename] = namespace.blank? ? file[:lang] : "#{namespace}.#{file[:lang]}"
    file[:path] = defined?(Rails) ? "#{Rails.root}/config/locales/#{file[:filename]}.yml" : ask_for_path
    if !File.exist?(file[:path]) && master
      log "File #{file[:path]} not found!"
      file[:path] = ask_for_path
    elsif !File.exist?(file[:path]) && !master
      log "File #{file[:filename]} not found! It will be created!"
      File.open(file[:path], 'w') { |f| f.write({ file[:lang] => {} }.to_yaml(line_width: -1)) }
    end
    file[:app_name] = Rails&.application&.class&.parent_name
    file
  end

  def self.ask_for_locales_path
    return "#{Rails.root}/config/locales" if defined?(Rails)
    log
    log 'Please type in the absolute path your locales-folder?'
    log 'e.g. /home/youruser/Documents/your-awesome-app/config/locales'
    path = [(print '# '), Readline.readline()][1]
    path
  end

  def self.ask_for_permission(master, count, slave = nil)
    log
    log 'Comparing Locales'
    log "Application: #{master[:app_name]}" if master[:app_name]
    log "Master Locale: #{master[:filename]}.yml" if master
    log "Slave Locale: #{slave[:filename]}.yml" if slave
    log "Difference: #{count} Entries"
    a = [(print "# Wanna do it? [Y/n]: "), Readline.readline()][1]
    a == 'y' || a == 'Y' || a == ''
  end

  def self.ask_for_task(tasks)
    log
    log 'Welcome, stranger!'
    log 'Are you lost in translation, too?'
    log 'Then I will be glad to help you one of the following tasks!'
    tasks.each_with_index do |task, i|
      log "#{i + 1} => #{task[:text]}"
    end
    log 'Soo ... what can I do for you? (Type in the Number)'
    id = [(print 'Task No.: '), Readline.readline()][1]
    tasks[(id.to_i - 1)]
  rescue
    puts "Cannot find task with id #{id}"
  end

  def self.ask_for_split(file, count)
    log
    log "Splitting File #{file}.yml"
    log "File-Count: #{count} new files will be create!"
    a = [(print "# Wanna do it? [Y/n]: "), Readline.readline()][1]
    a == 'y' || a == 'Y' || a == ''
  end

  def self.ask_for_sorting(file, app_name)
    log
    log "Application: #{app_name}" unless app_name.blank?
    log "Do you want the #{file}.yml to be sorted?"
    log 'Alphabetically & Recursive (ASC)'
    a = [(print "# [Y/n]: "), Readline.readline()][1]
    a == 'y' || a == 'Y' || a == ''
  end

  def self.ask_for_translation(value, new_structure, lang1, lang2, diff_count, diff_counter)
    system 'clear'
    log
    log "#{diff_counter} / #{diff_count}"
    new_s = new_structure.join('---')
    log "#{new_s} is missing"

    snippets = LostInTranslation.define_snippets.reverse
    snippets.each do |snippet|
      value = value.gsub(snippet.second, snippet.first)
    end

    puts "# In #{lang1.upcase}: #{value}"
    new_val = [(print "# In #{lang2.upcase}: "), Readline.readline()][1]
    new_val = (new_s + '---' + new_val) unless new_val.blank?
    new_val
  end

  def self.log(text = '')
    max_length = 80
    outside = 4
    puts '#' * max_length if text.blank?
    arr = text.scan(/.{1,#{max_length - outside}}/)
    arr.each do |str|
      fill_length = max_length - str.length - outside
      fill = ' ' * fill_length
      puts '# ' + str + fill + ' #'
    end
  end

  private

  def self.ask_for_path
    log
    log 'Please type in the absolute path your locale?'
    log 'e.g. /home/youruser/Documents/your-awesome-app/config/locales/de.yml'
    path = [(print '# '), Readline.readline()][1]
    path
  end
end
